@@ -27,14 +27,12 @@
                 structure(rep.int(i, nTheta),
                           names = paste(prefixLabels[i], termTools[[i]]$labels,
                                         sep = ""))
-            thetaID[[i]] <- structure(adj + seq(nTheta) - 1,
-                                      names = termTools[[i]]$labels)
             adj <- adj + nTheta
         }
         else if (all(common[b])) {
             designList <- lapply(unitLabels[b],
                                  function(x) class.ind(gnmData[[x]]))
-            
+
             ## get labels for all levels
             allLevels <- lapply(designList, colnames)
             labels <- unique(unlist(allLevels))
@@ -55,37 +53,36 @@
             factorAssign[b] <- lapply(i, function(x, nLevels, nm)
                                       structure(rep(x, nLevels), names = nm),
                                       nLevels, nm)
-            thetaID[b] <- rep(list(structure(adj + seq(nLevels) - 1,
-                                             names = nm)),
-                              length(designList))
             adj <- adj + nLevels
         }
         else {
             tmp <- model.matrix(terms(reformulate(c(0, unitLabels[b])),
                                       keep.order = TRUE), data = gnmData)
             tmpAssign <- attr(tmp, "assign") + !attr(tmp, "assign")[1]
-            nm <- paste(prefixLabels[b], colnames(tmp)[match[b] | sum(b) > 1],
+            tmpAssign <- which(b)[tmpAssign]
+            nm <- paste(prefixLabels[tmpAssign], colnames(tmp)[match[b] | sum(b) > 1],
                         sep = "")
-            tmpAssign <- structure(which(b)[tmpAssign],
-                                   names = nm)
+            names(tmpAssign) <- nm
             termTools[b] <- lapply(split(1:ncol(tmp), tmpAssign),
                                    function(i, M) M[, i , drop = FALSE], tmp)
             factorAssign[b] <- split(tmpAssign, tmpAssign)
-            thetaID[b] <- split(structure(adj + seq(length(tmpAssign)) - 1,
-                                          names = nm), tmpAssign)
             adj <- adj + length(tmpAssign)
         }
     }
 
     factorAssign <- unlist(factorAssign)
+    uniq <- !(duplicated(block) & common)[factorAssign]
     parLabels <- names(factorAssign)
     nTheta <- length(factorAssign)
+    thetaID <- numeric(nTheta)
+    thetaID[uniq] <- seq(sum(uniq))
+    thetaID[!uniq] <- thetaID[common[factorAssign] & uniq]
     nr <- dim(gnmData)[1]
     tmp <- seq(factorAssign) * nr
     first <- c(0, tmp[-nTheta])
-    firstX <- first[unlist(thetaID)]
+    firstX <- first[thetaID]
     last <- tmp - 1
-    lastX <- last[unlist(thetaID)] + 1
+    lastX <- last[thetaID] + 1
     nc <- tabulate(factorAssign)
     tmp <- cumsum(nc)
     a <- c(1, tmp[-nFactor] + 1)
@@ -95,24 +92,27 @@
         storage.mode(last) <- storage.mode(a) <-
         storage.mode(z) <- storage.mode(lt) <- "integer"
     baseMatrix <- matrix(1, nrow = nr, ncol = nTheta)
-    for (i in seq(termTools)) 
+    for (i in seq(termTools))
         if (is.matrix(termTools[[i]]))
             baseMatrix[, factorAssign == i] <- termTools[[i]]
     X <- baseMatrix
-    colID <- match(unlist(thetaID), unlist(thetaID))
-    if (any(duplicated(parLabels[unique(colID)]))){
-        parLabels[unique(colID)] <- make.unique(parLabels[unique(colID)])
+    colID <- match(thetaID, thetaID)
+    thetaID <- split(thetaID, factorAssign)
+    names(thetaID) <- varLabels
+    if (any(duplicated(parLabels[uniq]))){
+        parLabels[uniq] <- make.unique(parLabels[uniq])
         warning("Using make.unique() to make default parameter labels unique",
                 call. = FALSE)
     }
     colnames(X) <- parLabels
-    X <- X[, unique(colID), drop = FALSE]
+    X <- X[, uniq, drop = FALSE]
 
     thetaClassID <- structure(classID[factorAssign], names = parLabels)
-    theta <- rep(NA, length(factorAssign))
+    theta <- rep(NA, nTheta)
     for (i in blockID) {
         b <- block == i
-        if (sum(b) == 1 && !is.null(termTools[[which(b)]]$start)){
+        if (sum(b) == 1 && is.list(termTools[[which(b)]]) &&
+            !is.null(termTools[[which(b)]]$start)){
             theta[unlist(thetaID[b])] <- termTools[[which(b)]]$start
             thetaClassID[unlist(thetaID[b])][is.na(termTools[[which(b)]]$start)] <-
                 "Linear"
@@ -120,8 +120,8 @@
     }
     names(theta) <- parLabels
 
-    thetaClassID <- thetaClassID[unique(colID)]
-    theta <- theta[unique(colID)]
+    thetaClassID <- thetaClassID[uniq]
+    theta <- theta[uniq]
 
     for (i in seq(attr(modelTerms, "predictor"))) {
         if (!is.null(attr(modelTerms, "start")[[i]])) {
@@ -129,7 +129,7 @@
             theta[termID] <- attr(modelTerms, "start")[[i]](theta[termID])
         }
     }
-    
+
     if (x || termPredictors) {
         termAssign <- attr(modelTerms, "assign")[factorAssign]
         if (attr(modelTerms, "intercept"))
@@ -154,7 +154,7 @@
         }
         prodList
     }
-    
+
     predictor <- function(varPredictors, term = FALSE) {
         if (term)
             sapply(attr(modelTerms, "predictor"), eval,
@@ -164,11 +164,11 @@
     }
 
     gnmData <- lapply(gnmData[, !names(gnmData) %in% varLabels, drop = FALSE],
-                      drop) 
+                      drop)
     e <- do.call("bquote",
                  list(sumExpression(attr(modelTerms, "predictor")), gnmData))
     varDerivs <- lapply(varLabels, deriv, expr = e)
-    
+
 
     commonAssign <- factorAssign[colID]
     nCommon <- table(commonAssign[!duplicated(factorAssign)])
@@ -177,19 +177,18 @@
     nCommon <- as.integer(nCommon[as.character(tmpID)])
     if (any(NonlinID | classID))
         specialVarDerivs <- deriv(e, varLabels[(NonlinID | classID)])
-    convIND <- unique(colID)
+    convID <- colID[uniq]
     vID <- cumsum(c(1, nCommon))[seq(nCommon)]
-    
+
     localDesignFunction <- function(theta, varPredictors, ind = NULL) {
-        #browser()
         if (!any(common)) {
             if (!is.null(ind)){
-                i1 <- convIND[ind]
+                i1 <- convID[ind]
                 tmpID <- commonAssign[i1]
             }
-                                       
+
             for (i in tmpID) {
-                fi <- unique(factorAssign[commonAssign == i]) 
+                fi <- unique(factorAssign[commonAssign == i])
                 if (is.null(ind)){
                     i1  <- a[fi][1]
                     i2 <- z[fi][1]
@@ -233,10 +232,10 @@
                       nr, nCommon, max(nCommon), PACKAGE = "gnm")
             }
             else {
-                i1 <- convIND[ind]
+                i1 <- convID[ind]
                 fi <- unique(factorAssign[commonAssign == commonAssign[i1]])
                 v <- list()
-                for(j in fi) 
+                for(j in fi)
                     v[[j]] <- attr(eval(varDerivs[[j]], varPredictors),
                                    "gradient")
                 .Call("single", baseMatrix, as.double(unlist(v[fi])),
